﻿ using System;
using System.Collections.Generic;
using System.Collections;
using System.IO;
using UnityEngine;
using UnityEngine.UI;
using System.Linq;
using TMPro;
using UnityEngine.Events;

namespace Mod
{
	public class Mod : MonoBehaviour
	{
		public static void Main()
		{
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Interphase Human",
					//NameToOrderByOverride = "Interphase Human",
					DescriptionOverride = "Phase into an alternate space to avoid damage.\n<b>Press on the head to turn on/off the ability</b>",
					CategoryOverride = ModAPI.FindCategory("Entities"),
					ThumbnailOverride = ModAPI.LoadSprite("Thumbnail/Interphase Human.png"),
					AfterSpawn = (Instance) =>
					{
						var skin = ModAPI.LoadTexture("Sprites/Interphase Human.png");
						var flesh = ModAPI.LoadTexture("Sprites/flesh layer.png");
						var bone = ModAPI.LoadTexture("Sprites/bone layer.png");
						var person = Instance.GetComponent<PersonBehaviour>();
						person.SetBodyTextures(skin, flesh, bone);

						Transform head = Instance.transform.Find("Head");
	
					//PhaseShiftBehavior
						PhaseShiftBehavior phase = Instance.transform.Find("Head").gameObject.AddComponent<PhaseShiftBehavior>();
						phase.EntryClip = ModAPI.LoadSound("Sfx/PhaseEntry.mp3");
						phase.LoopClip = ModAPI.LoadSound("Sfx/PhaseNone.mp3");
						phase.ExitClip = ModAPI.LoadSound("Sfx/PhaseExit.mp3");
						phase.childs = new GameObject[]
						{
						};					

					}
				}
			);
		}
	}
	public class PhaseShiftBehavior : MonoBehaviour
	{
		//service
		public bool inPhase {private set; get;} = false;
		public bool handsChecking {private set; get;} = false;
		
		[SkipSerialisation]
		private AudioSource Audio;
		[SkipSerialisation]
		private AudioSource LoopAudio;
		//private Vector3 DefaultGripPos;
		//private float DefaultFlammability;
		private GameObject[] Hands;
		
		//editable		
		[SkipSerialisation]
		public AudioClip EntryClip;
		[SkipSerialisation]
		public AudioClip LoopClip;
		[SkipSerialisation]
		public AudioClip ExitClip;

		public GameObject[] childs;

		private void Start()
		{
			Audio = gameObject.AddComponent<AudioSource>();
			LoopAudio = gameObject.AddComponent<AudioSource>();
			gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
			LoopAudio.loop = true;
			LoopAudio.clip = LoopClip;
			Hands = new GameObject[]
			{
				gameObject.transform.parent.transform.Find("BackArm").Find("LowerArm").gameObject,
				gameObject.transform.parent.transform.Find("FrontArm").Find("LowerArmFront").gameObject 
			};
			foreach (GameObject hand in Hands)
			{
				hand.AddComponent<PhaseShiftHandBehavior>().LoadData(gameObject.GetComponent<PhaseShiftBehavior>());
			}
		}		
		private void Use()
		{
			if (gameObject.GetComponent<LimbBehaviour>().Person.IsAlive())
			{
				inPhase = !inPhase;
				//Debug.Log("<b>PhaseShiftBehavior :</b> inPhase = " + inPhase);
				if (inPhase)
				{
					Audio.PlayOneShot(EntryClip);
					LoopAudio.Play();
					foreach (var body in GetComponent<LimbBehaviour>().Person.Limbs)
					{		
						IntoThePhase(body.gameObject);
					}					
					foreach (GameObject child in childs)
					{
						IntoThePhase(child);
					}	
					foreach (GameObject hand in Hands)
					{						
						if (hand.GetComponent<GripBehaviour>().CurrentlyHolding != null)
						{
							GameObject Object = hand.GetComponent<GripBehaviour>().CurrentlyHolding.gameObject;

							IntoThePhase(Object);	
							
							if(Object.GetComponent<FirearmBehaviour>() != null)
							{
								Object.GetComponent<FirearmBehaviour>().IgnoreUse = true;
							}
							if(Object.GetComponent<ProjectileLauncherBehaviour>() != null)
							{
								Object.GetComponent<ProjectileLauncherBehaviour>().enabled = false;
							}
							try
							{
								foreach (Transform child in Object.transform)
								{
									IntoThePhase(child.gameObject);
								}
							}
							catch
							{
								continue;
							}
						}
					}
					handsChecking = true;							
				}
				else
				{
					Audio.PlayOneShot(ExitClip);
					LoopAudio.Stop();
					foreach (var body in GetComponent<LimbBehaviour>().Person.Limbs)
					{
						OutOfPhase(body.gameObject);
					}
					foreach (GameObject child in childs)
					{
						OutOfPhase(child);
					}
					foreach (GameObject hand in Hands)
					{						
						if (hand.GetComponent<GripBehaviour>().CurrentlyHolding != null)
						{
							GameObject Object = hand.GetComponent<GripBehaviour>().CurrentlyHolding.gameObject;	

							OutOfPhase(Object);	
							if(Object.GetComponent<FirearmBehaviour>() != null)
							{
								Object.GetComponent<FirearmBehaviour>().IgnoreUse = false;
							}
							if(Object.GetComponent<ProjectileLauncherBehaviour>() != null)
							{
								Object.GetComponent<ProjectileLauncherBehaviour>().enabled = true;
							}
							try
							{
								foreach (Transform child in Object.transform)
								{
									OutOfPhase(child.gameObject);
								}
							}
							catch
							{
								continue;
							}
						}
					}	
					handsChecking = true;		
				}
			}
			else
			{
				LoopAudio.Stop();
			}
		}
		private void IntoThePhase(GameObject Object)
		{
			Object.GetComponent<SpriteRenderer>().color = new Color(0.5f, 0.5f, 0.5f, 0.5f);
			Object.gameObject.layer = 10;	
		}
		private void OutOfPhase(GameObject Object) 
		{
			Object.GetComponent<SpriteRenderer>().color = new Color(1f, 1f, 1f, 1f);
			Object.gameObject.layer = LayerMask.NameToLayer("Objects");
		}
	}	
	public class PhaseShiftHandBehavior : MonoBehaviour
	{
		[SkipSerialisation]
		private PhaseShiftBehavior psb;
		[SkipSerialisation]
		private GripBehaviour gb;

		public void LoadData(PhaseShiftBehavior loadedPSB) 
		{
			psb = loadedPSB;
			gb = gameObject.GetComponent<GripBehaviour>();
		}
		private void Update() 
		{
			if (psb.inPhase && psb.handsChecking)
			{
				if(gb.isHolding)
				{
					if (gb.CurrentlyHolding.gameObject.layer != 10)
					{
						gb.DropObject();
					}
				}
			}
		}
	}
 }